@@ -2,22 +2,23 @@ from aiohttp import web
 import asyncio
 from collections import namedtuple
 from copy import deepcopy
-from flask import Flask, request
 from functools import partial
 from inspect import isawaitable
 import json
 from typing import Callable
 from types import FunctionType
+import logging
+import os
+from bottle import Bottle
 
-from i2.errors import InputError
+from i2.errors import InputError, DataError, AuthorizationError
 
-from py2http.config import mk_config, FLASK, AIOHTTP
+from py2http.config import mk_config, FLASK, AIOHTTP, BOTTLE
 from py2http.default_configs import default_configs
 from py2http.openapi_utils import add_paths_to_spec, mk_openapi_path, mk_openapi_template
 from py2http.schema_tools import mk_input_schema_from_func, mk_output_schema_from_func
 from py2http.util import TypeAsserter
 
-
 def method_not_found(method_name):
     raise web.HTTPNotFound(text=json.dumps({'error': f'method {method_name} not found'}),
                            content_type='application/json')
@@ -39,20 +40,33 @@ def func_copy(func) -> Callable:
 
 def mk_route(func, **configs):
     """
-    Generate an aiohttp route object and an OpenAPI path specification for a function
+    Generate an route object and an OpenAPI path specification for a function
 
     :param func: The function
 
     :Keyword Arguments: The configuration settings
     """
 
+    def get_input_args_and_kwargs(inputs):
+        input_args = ()
+        input_kwargs = {}
+        if isinstance(inputs, dict):
+            input_kwargs = inputs
+        elif isinstance(inputs, list):
+            input_args = tuple(inputs)
+        elif isinstance(inputs, tuple):
+            input_args = inputs[0]
+            input_kwargs = inputs[1]
+        return input_args, input_kwargs
+
     # TODO: perhaps collections.abc.Mapping initialized with func, configs, etc.
     config_for = partial(mk_config, func=func, configs=configs, defaults=default_configs)
-    framework = config_for('framework')
+    framework = _get_framework(configs, default_configs)
     input_mapper = config_for('input_mapper')
     output_mapper = config_for('output_mapper')
     error_handler = config_for('error_handler')
     header_inputs = config_for('header_inputs', type=dict)
+    logger = config_for('logger')
 
     exclude_request_keys = header_inputs.keys()
     request_schema = getattr(input_mapper, 'request_schema', None)
@@ -66,66 +80,92 @@ def mk_route(func, **configs):
                                   'response_schema',
                                   mk_output_schema_from_func(func))
 
-    async def handle_request(req):
-        input_kwargs = {}
-        try:
-            inputs = input_mapper(req, request_schema)
-            if isawaitable(inputs):  # Pattern: pass-on async property
-                inputs = await inputs
-            if isinstance(inputs, dict):
-                input_args = ()
-                input_kwargs = inputs
-            elif isinstance(inputs, list):
-                input_args = tuple(inputs)
-                input_kwargs = {}
-            elif isinstance(inputs, tuple):
-                input_args = inputs[0]
-                input_kwargs = inputs[1]
+    def handle_error(func):
+        def handle_request(req):
             try:
-                raw_result = func(*input_args, **input_kwargs)
-            except TypeError as error:
-                raise InputError(str(error))
-            if isawaitable(raw_result):  # Pattern: pass-on async property
-                raw_result = await raw_result
+                return func(req)
+            except (DataError, AuthorizationError, InputError) as error:
+                if logger:
+                    level = logging.INFO if logger.getEffectiveLevel() >= logging.INFO else logging.DEBUG
+                    exc_info = level == logging.DEBUG
+                    logger.log(level, error, exc_info=exc_info)
+                return error_handler(error)
+            except Exception as error:
+                if logger:
+                    logger.exception(error)
+                return error_handler(error)
 
-            final_result = output_mapper(raw_result, input_kwargs)
-            if isawaitable(final_result):
-                final_result = await final_result
-            if framework == AIOHTTP and not isinstance(final_result, web.Response):
-                final_result = web.json_response(final_result)
-            return final_result
-        except Exception as error:
-            return error_handler(error, input_kwargs)
+        return handle_request
 
-    # TODO: Align config keys and variable names
+    @handle_error
+    def sync_handle_request(req):
+        if framework == BOTTLE:
+            req.get_json = lambda *x: req.json
+        inputs = input_mapper(req, request_schema)
+        input_args, input_kwargs = get_input_args_and_kwargs(inputs)
+        try:
+            raw_result = func(*input_args, **input_kwargs)
+        except TypeError as error:
+            raise InputError(str(error))
+
+        return output_mapper(raw_result, input_kwargs)
+
+    @handle_error
+    async def aiohttp_handle_request(req):
+        inputs = input_mapper(req, request_schema)
+        if isawaitable(inputs):  # Pattern: pass-on async property
+            inputs = await inputs
+        input_args, input_kwargs = get_input_args_and_kwargs(inputs)
+        try:
+            raw_result = func(*input_args, **input_kwargs)
+        except TypeError as error:
+            raise InputError(str(error))
+        if isawaitable(raw_result):  # Pattern: pass-on async property
+            raw_result = await raw_result
+
+        final_result = output_mapper(raw_result, input_kwargs)
+        if isawaitable(final_result):
+            final_result = await final_result
+        if not isinstance(final_result, web.Response):
+            final_result = web.json_response(final_result)
+        return final_result
+
+    #  TODO: Align config keys and variable names
     valid_http_methods = {'get', 'put', 'post', 'delete'}  # outside function
     http_method = config_for('http_method')  # read
     assert isinstance(http_method, str)  # validation
     http_method = http_method.lower()  # normalization
     assert http_method in valid_http_methods  # validation
 
-    def mk_framework_route(http_method, path, method_name, handler):
+    def mk_framework_route(http_method, path, method_name):
         if framework == AIOHTTP:
             web_mk_route = getattr(web, http_method)
-            return web_mk_route(path, handler)
-        if framework == FLASK:
-            def sync_handle_request():
-                loop = asyncio.new_event_loop()
-                asyncio.set_event_loop(loop)
-                result = loop.run_until_complete(handler(request))
+            return web_mk_route(path, aiohttp_handle_request)
+        else:
+            if framework == FLASK:
+                from flask import request
+            elif framework == BOTTLE:
+                from bottle import request
+
+            def handle_request(*args):
+                result = sync_handle_request(request)
+                if isawaitable(result):
+                    loop = asyncio.new_event_loop()
+                    asyncio.set_event_loop(loop)
+                    result = loop.run_until_complete(result)
                 return result
-            sync_handle_request.path = path
-            sync_handle_request.http_method = http_method
-            sync_handle_request.method_name = method_name
-            return sync_handle_request
-        return None
+
+            handle_request.path = path
+            handle_request.http_method = http_method
+            handle_request.method_name = method_name
+            return handle_request
 
     # TODO: Make func -> path a function (not hardcoded)
     # TODO: Make sure that func -> path MAPPING is known outside (perhaps through openapi)
     method_name = config_for('name') or func.__name__
     path = config_for('route') or f'/{method_name}'
 
-    route = mk_framework_route(http_method, path, method_name, handle_request)
+    route = mk_framework_route(http_method, path, method_name)
 
     path_fields = dict({'x-method_name': method_name}, **extra_path_info(func))
     openapi_path = mk_openapi_path(path,
@@ -155,32 +195,43 @@ def run_http_service(funcs, **configs):
     """
     app = mk_http_service(funcs, **configs)
     port = configs.get('port', app.dflt_port)
-    framework = mk_config('framework', None, configs, default_configs)
+    framework = _get_framework(configs, default_configs)
     if framework == FLASK:
         return run_flask_service(app, port)
-    if framework == AIOHTTP:
-        return run_aiohttp_service(app, port)
+    if framework == BOTTLE:
+        return run_bottle_service(app, port)
+    return run_aiohttp_service(app, port)
 
 
 def run_flask_service(app, port):
     app.run(port=port, debug=True)
 
 
+def run_bottle_service(app, port):
+    app.run(port=port, debug=True)
+
+
 def run_aiohttp_service(app, port):
     web.run_app(app, port=port)
 
 
 def mk_http_service(funcs, **configs):
     routes, openapi_spec = mk_routes_and_openapi_specs(funcs, configs)
-    framework = mk_config('framework', None, configs, default_configs)
+    framework = _get_framework(configs, default_configs)
     if framework == FLASK:
         return mk_flask_service(routes, openapi_spec, **configs)
+    if framework == BOTTLE:
+        return mk_bottle_service(routes, openapi_spec, **configs)
     return mk_aiohttp_service(routes, openapi_spec, **configs)
 
 
 def mk_flask_service(routes, openapi_spec, **configs):
+    from flask import Flask
     app_name = mk_config('app_name', None, configs, default_configs)
     app = Flask(app_name)
+    middleware = mk_config('middleware', None, configs, default_configs)
+    if middleware:
+        app = middleware(app)
     for route in routes:
         app.add_url_rule(route.path, route.method_name, route, methods=[route.http_method.upper()])
     app.openapi_spec = openapi_spec
@@ -198,6 +249,20 @@ def mk_aiohttp_service(routes, openapi_spec, **configs):
     return app
 
 
+def mk_bottle_service(routes, openapi_spec, **configs):
+    app = Bottle(catchall=False)
+    plugins = mk_config('plugins', None, configs, default_configs)
+    if plugins:
+        for plugin in plugins:
+            app.install(plugin)
+    for route in routes:
+        # print(f'Mounting route: {route.path} {route.http_method.upper()}')
+        app.route(route.path, route.http_method.upper(), route, route.method_name)
+    app.openapi_spec = openapi_spec
+    app.dflt_port = mk_config('port', None, configs, default_configs)
+    return app
+
+
 # TODO: Make signature explicit instead of using configs
 #   What it needs from config is only: openapi_spec and a header_inputs
 #   Make the user of this function get those from general configs
@@ -220,12 +285,34 @@ def mk_routes_and_openapi_specs(funcs, configs):
     return routes, openapi_spec
 
 
-def run_many_services(apps, **configs):
-    app = web.Application()
+def run_many_services(apps, run_now=False, **configs):
+    framework = _get_framework(configs, default_configs)
+    if framework == BOTTLE:
+        return run_many_bottle_services(apps, run_now=run_now, **configs)
+    return run_many_aiohttp_services(apps, run_now=run_now, **configs)
+
+
+def run_many_bottle_services(apps, run_now=False, **configs):
+    from bottle import Bottle, run
+    parent_app = Bottle(catchall=False)
     for route, subapp in apps.items():
-        app.add_subapp(route, subapp)
-    port = mk_config('port', None, configs, default_configs)
-    web.run_app(app, port=port)
+        parent_app.mount(route, subapp)
+    if run_now:
+        port = mk_config('port', None, configs, default_configs)
+        run(parent_app, port=port)
+    else:
+        return parent_app
+
+
+def run_many_aiohttp_services(apps, run_now=False, **configs):
+    parent_app = web.Application()
+    for route, subapp in apps.items():
+        parent_app.add_subapp(route, subapp)
+    if run_now:
+        port = mk_config('port', None, configs, default_configs)
+        web.run_app(parent_app, port=port)
+    else:
+        return parent_app
 
 
 ##########################################################################################
@@ -277,3 +364,9 @@ def mk_config_nt(keys, *args, **kwargs):
     # input_mapper = mk_config('input_mapper', func, configs, default_configs)
     # output_mapper = mk_config('output_mapper', func, configs, default_configs)
     # header_inputs = mk_config('header_inputs', func, configs, default_configs)
+
+
+def _get_framework(configs, default_configs):
+    framework = mk_config('framework', None, configs, default_configs)
+    os.environ['PY2HTTP_FRAMEWORK'] = framework
+    return framework
